###############################################################################
 #  Copyright 2009 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
###############################################################################

class GLVertexShader
	attr_accessor :shader

	def initialize(source)
		@shader = glCreateShader(GL_VERTEX_SHADER)
		@source = source

		glShaderSource(@shader, @source)
		glCompileShader(@shader)

		puts "Vertex Shader: #{glGetShaderInfoLog(@shader)}" unless (glGetShaderiv(@shader, GL_COMPILE_STATUS) == true)
	end
end

class GLFragmentShader
	attr_accessor :shader

	def initialize(source)
		@shader = glCreateShader(GL_FRAGMENT_SHADER)
		@source = source

		glShaderSource(@shader, @source)
		glCompileShader(@shader)

		puts "Fragment Shader: #{glGetShaderInfoLog(@shader)}\n" unless (glGetShaderiv(@shader, GL_COMPILE_STATUS) == true)
	end
end

class GLShaderProgram
	boolean_accessor :ok

	def initialize(opts)
		#puts '==================== Compiling ===================='

		@vertex_shader = GLVertexShader.new(opts[:vertex_shader_source])
		@fragment_shader = GLFragmentShader.new(opts[:fragment_shader_source])

		@program = glCreateProgram
		glAttachShader(@program, @vertex_shader.shader)
		glAttachShader(@program, @fragment_shader.shader)

		glLinkProgram(@program)

		@uniform_to_location_hash = {}

		if (glGetProgramiv(@program, GL_LINK_STATUS) == true)
			ok!
			#puts 'Compilation Successful'
		else
			puts "Shader Program:\n#{glGetProgramInfoLog(@program)}\n"
		end
	end

	def using
		return yield self unless ok?

		glUseProgram(@program)
		yield self
		glUseProgram(0)
	end

	def find_uniform(name)
		location = @uniform_to_location_hash[name]
		return location if location

		location = glGetUniformLocation(@program, name)
		puts "shader uniform '#{name}' not found" if location == -1
		@uniform_to_location_hash[name] = location
		return location
	end

	def set_3f(name, value)
		glUniform3f(find_uniform(name), *(value.to_a))
	end

	def set_f(name, value)
		glUniform1f(find_uniform(name), value)
	end

	def set_int(name, value)
		glUniform1i(find_uniform(name), value)
	end
end

class ActorEffectShader < ActorEffect
	virtual

	title				'Shader'
	description ''

	def after_load
		super
		@program = GLShaderProgram.new(:vertex_shader_source => vertex_shader_source, :fragment_shader_source => fragment_shader_source)
	end

	def vertex_shader_source
		"
		void main(void)
		{
			gl_Position = ftransform();
			gl_TexCoord[0] = gl_TextureMatrix[0] * gl_MultiTexCoord0;
			gl_FrontColor = gl_Color;
		}
		"
	end

	def fragment_shader_source
		"
		uniform sampler2D tex;

		void main(void)
		{
			gl_FragColor = gl_Color * texture2D(tex, gl_TexCoord[0].st);
		}
		"
	end

	def set_uniforms(program)
		program.set_f('time', $env[:time] % 1.0)
	end

	def render
		@program.using { |program|
			set_uniforms(program)
			yield
		}
	end
end
