#!/usr/bin/ruby

require 'gst'
require 'utils/osc'
require 'utils/ip_socket'		# for IPSocket#set_reuse_address_flag
require 'utils/gstreamer_audio_player'

require 'constants'		# for MESSAGE_BUS_IP and MESSAGE_BUS_PORT

################################################################################
#
# main()
#
################################################################################

#
# UDP Socket for sending OpenSoundControl
#
$socket = UDPSocket.new.set_reuse_address_flag
$socket.bind(Socket::INADDR_ANY, 0)
$socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_BROADCAST, true)

#
# AudioPlayer
#
$player = GStreamerAudioPlayer.new
$player.on_position_changed {
	# calculate progress as 0.0..1.0
	progress = $player.position_in_ms.to_f / $player.duration_in_ms.to_f

	# create OSC message
	message = OSC::Message.new('Audio / Progress', 'f', progress)

	# send it out
	$socket.send(message.encode, 0, MESSAGE_BUS_IP, MESSAGE_BUS_PORT)		# 0 means calculate the length
}

#
# start audio file
#
$player.open(ARGV[0])
$player.play

#
# a GLib mainloop runs the show
#
$mainloop = GLib::MainLoop.new(GLib::MainContext.default, true)
$mainloop.run
