 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

# Stage is responsible for "setting the stage", that is, preparing OpenGL for
# actor rendering.  It also draws the background.

require 'parent_user_object', 'director_effect', 'taggable', 'drawing'

class Director < ParentUserObject
	include Taggable
	include Drawing

	ACTOR_RADIUS = 0.5 		# (used by children)

	def self.clear_to_black(clear_bits = GL::COLOR_BUFFER_BIT)
		GL.ClearColor(0,0,0,0)
		GL.Clear(clear_bits)
	end

	###################################################################
	# Object-level functions
	###################################################################
	attr_accessor :effects
	empty_method :render
	attr_accessor :x, :y

	def z
		0.0
	end

	def to_yaml_properties
		tag_instance_variables + super
	end

	def after_load
		set_default_instance_variables(:title => 'New Director', :x => 0.0, :y => 0.0)
		super
		after_load_tag_class_registration
	end

	def before_delete
		clear_tags
		super
	end

	###################################################################
	# Render
	###################################################################
	def do_render
		user_object_try { render_scene_recursive }
	end

private

	def render_scene_recursive(effect_index = 0, options = {})
		if (effect_index and effect = effects[effect_index])
			if !effect.usable?
				render_scene_recursive(effect_index + 1, options)		# Skip this effect
			else
				$engine.user_object_try(effect) {
					effect.resolve_settings
					effect.render {
						render_scene_recursive(effect_index + 1, options)
					}
				}
			end
		else
			# reached bottom of list inside yields
		end
	end
end
