#include "input-joystick.h"
#include <stdio.h>
#include <stdlib.h>
#include <cstring>

#include "utils.h"

void joystick_init()
{
	SDL_Quit();
	SDL_Init(SDL_INIT_JOYSTICK);
}

int joystick_count()
{
	SDL_NumJoysticks();
}

class InputJoystick;

InputJoystick* joystick_open_by_index(int index)
{
	return new InputJoystick(index);
}

InputJoystick::InputJoystick(int index)
	: m_device_name(SDL_JoystickName(index))
{
	m_joystick = SDL_JoystickOpen(index);

	// axis
	m_axis_count = SDL_JoystickNumAxes(m_joystick);
	m_axis_values = (int*)calloc(m_axis_count, sizeof(int));
	m_axis_limits = (TLimits*)calloc(m_axis_count, sizeof(TLimits));		// 0 for min and max is fine for joystick axes

	// buttons
	m_button_count = SDL_JoystickNumButtons(m_joystick);
	m_button_values = (bool*)calloc(m_button_count, sizeof(bool));
}

const char* InputJoystick::device_type()
{
	return "Joystick";
}

const char* InputJoystick::device_name()
{
	return (const char*)(m_device_name.c_str());
}

bool InputJoystick::update()
{
	SDL_JoystickUpdate();		// NOTE: this shouldn't go here, won't work for 2+ joysticks..?

	//
	// Update buttons
	//
	for(int button_index=0; button_index<m_button_count ; button_index++) {
		if(SDL_JoystickGetButton(m_joystick, button_index) == SDL_PRESSED) {
			// Newly pressed?
			if(m_button_values[button_index] == false) {
				m_button_values[button_index] = true;

				send_integer(button_names[button_index + 1], 1);
			}
		}
		else {
			// Newly released?
			if(m_button_values[button_index] == true) {
				m_button_values[button_index] = false;

				send_integer(button_names[button_index + 1], 0);
			}
		}
	}

	//
	// Update axes
	//
	for(int axis_index=0; axis_index<m_axis_count ; axis_index++) {
		int axis_value = SDL_JoystickGetAxis(m_joystick, axis_index);

		// Moved?
		if(axis_value != m_axis_values[axis_index]) {
			float value = scale_and_expand_limits(axis_value, &(m_axis_limits[axis_index]));

			send_float(axis_names[axis_index + 1], value);

			m_axis_values[axis_index] = axis_value;		// NOTE: this saves the raw integer axis_value
		}
	}
	return true;
}

void InputJoystick::sleep()
{
	usleep(5 * 1000);
}

InputJoystick::~InputJoystick()
{
}
