###############################################################################
 #  Copyright 2009 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
###############################################################################

class ActorEffectShaderFractalMandelbrot < ActorEffectShader
	title				'Shader Fractal Mandelbrot'
	description ''

	setting 'c_x', :float, :default => 0.0..1.0
	setting 'c_y', :float, :default => 0.0..1.0
	setting 'scale', :float, :default => 0.0..1.0

	setting 'image_y', :float, :range => 0.0..1.0, :default => 0.0..1.0
	setting 'max_depth', :integer, :range => 1..1000, :default => 20..40

	def set_uniforms(program)
		program.set_f('c_x', c_x)
		program.set_f('c_y', c_y)
		program.set_f('scale', scale)
		program.set_f('image_y', image_y)
		program.set_int('max_depth', max_depth)
		super
	end

	def fragment_shader_source
		"
		uniform float time;
		uniform sampler2D tex;

		uniform float c_x;
		uniform float c_y;
		uniform float scale;
		uniform float image_y;
		uniform int max_depth;

		void main(void)
		{
			vec2 z, c;

			c.x = 1.3333 * (gl_TexCoord[0].x - 0.5) * scale - c_x;
			c.y = (gl_TexCoord[0].y - 0.5) * scale - c_y;

			int i;
			for(i=0; i<max_depth; i++) {
					float x = (z.x * z.x - z.y * z.y) + c.x;
					float y = (z.y * z.x + z.x * z.y) + c.y;

					if((x * x + y * y) > 4.0) break;
					z.x = x;
					z.y = y;
			}

			gl_FragColor = texture2D(tex, vec2((i == max_depth ? 0.0 : float(i)) / 100.0, image_y));
		}
		"
	end
end
