 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class Factory
	def character_image(character)
		@canvas ||= CairoCanvas.new(256,256)

		@images_for_characters ||= Hash.new { |hash, key|
			hash[key] = generate_character_image(key)
		}

		@images_for_characters[character]
	end

	def generate_character_image(character)
		image = Image.new

		@canvas.using { |context|
			context.save
				context.set_source_rgba(0.0, 0.0, 0.0, 0.0)		# NOTE: alpha 0.0
				context.set_operator(:source)
				context.paint

				# Default Cairo coordinates has 0,0 in the upper left, with 1 unit translating to 1 pixel
				# Scale it so that the whole canvas goes 0.0 -> 1.0 horizontally and 0.0 -> 1.0 vertically
				context.scale(@canvas.width, @canvas.height)		# NOTE: height is multiplied by -1 to flip the canvas vertically, so bigger numbers go up (cartessian plane)

				# Move cursor to center
				context.translate(0.5, 0.5)

				context.set_source_rgba(1,1,1,1)
				context.set_antialias(Cairo::ANTIALIAS_GRAY)		# or DEFAULT or GRAY or SUBPIXEL or NONE

				context.select_font_face("Arial Black", Cairo::FONT_SLANT_NORMAL, Cairo::FONT_WEIGHT_NORMAL) # Cairo::FONT_WEIGHT_BOLD)		#
				context.set_font_size(1.0)

				extents = context.text_extents(character)

				context.move_to(-0.5 - extents.x_bearing + ((1.0 - extents.width)/2.0), 0.25)
				context.show_text(character)
			context.restore
		}

		image.from_bgra8(@canvas.string_data, @canvas.width, @canvas.height)
		image
	end
end

class ActorEffectTextChildren < ActorEffect
	title				"Text Children"
	description "Apply text to children, one letter per child"

	setting :text, :string

	def deep_clone(*args)
		@factory = nil
		super(*args)
	end

	def render
		@factory ||= Factory.new

		character = text[child_index]
		return yield unless character

#puts character.chr

		image = @factory.character_image(character.chr)
		return yield unless image

		image.using {
			yield
		}
	end
end
