 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class DirectorEffectActorCycle < DirectorEffect
	title				'Actor Cycle'
	description ''

	setting 'tag', :actors
	setting 'forwards', :event
	setting 'backwards', :event
	setting 'crossfade_time', :timespan

	def render
		return yield if tag.count == 0

		#	TODO: this does not belong in render!
		@current_index ||= 0
		@desired_index ||= 0
		@desired_index += 1 if forwards.now?
		@desired_index -= 1 if backwards.now?


		if @current_index == @desired_index
			tag.one(@current_index) { |a| a.do_render }
		else
			#puts "#{@current_index} => #{@desired_index}"

			# Start a timer, if one isn't running
			@switch_start_time ||= $env[:time]

			progress = crossfade_time.progress_since(@switch_start_time)
			#puts sprintf("%0.02f", progress)

			if @desired_index > @current_index
				# We are moving forwards
				# Draw new one first, behind old one
				with_env(:enter, progress) {
					tag.one(@current_index+1) { |a| a.do_render }
				}

				if progress == 1.0
					@switch_start_time = nil
					@current_index += 1
				else
					with_env(:exit, progress) {
						tag.one(@current_index) { |a| a.do_render }
					}
				end
			else
				# We are moving backwards
				# same: Draw new one first, behind old one
				with_env(:enter, progress) {
					tag.one(@current_index-1) { |a| a.do_render }
				}

				if progress == 1.0
					@switch_start_time = nil
					@current_index -= 1
				else
					with_env(:exit, progress) {
						tag.one(@current_index) { |a| a.do_render }
					}
				end
			end
		end

		yield
	end
end
