 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class DirectorEffectActorPlant < DirectorEffect
	virtual	# deprecated (put this on the actor level)

	title				'Actor Plant'
	description ''

	setting 'tag', :actors

	setting 'number', :integer, :range => 1..10, :default => 1..2
	setting 'size', :float, :range => 0.0..100.0, :default => 1.0..2.0
	setting 'progress', :float, :range => 0.0..1.0, :default => 0.0..1.0
	setting 'curve', :float, :range => -0.25..0.25, :default => 0.0..0.25
	setting 'spacing', :float, :range => -1.0..1.0, :default => 0.0..1.0

	def render
		if progress > 0.0
			scale(size) {
				current_index, current_progress = progress.index_and_progress_to(number)
				rotate(curve) {
					render_leaf(0, current_progress, current_index+1, curve, spacing)
				}
			}
		end
		yield
	end

	def render_leaf(index, leaf_progress, total, curve, spacing)
		# for speed, do this before calling: return unless index < total

		tag.one(index) { |a|
			# grown, growing, or not yet started?
			if index < total-1
				# grown
				with_enter_and_exit(1.0, 0.0) { a.do_render }

				# next leaf
				slide(spacing) {
					rotate(curve) {
						render_leaf(index+1, leaf_progress, total, curve, spacing)
					}
				}
			elsif index == total-1
				# growing
				with_enter_and_exit(leaf_progress, 0.0) { a.do_render }
			else
				# not yet started
			end
		}
	end
end
