 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

require 'struct-stack'

class DirectorEffectActorSpawnerXYZ < DirectorEffect
	virtual	# deprecated

	title				'Actor Spawner XYZ'
	description ''

	setting 'actors', :actors
	setting 'spawn', :event
	setting 'lifetime', :timespan

	setting 'spawn_x', :float, :default => 0.0..1.0
	setting 'spawn_y', :float, :default => 0.0..1.0
	setting 'spawn_z', :float, :default => 0.0..1.0

	Character = StructStack.new(:actor, :birth_time, :x, :y, :z)

	def render
		@stack ||= []		# TODO: ugly
		@child_index_offset ||= 0

		# TODO: This should be in a tick method and one line of code
		if spawn.now?
			actors.one(@child_index_offset) { |actor|
				@stack << Character.pop(actor, $env[:time], spawn_x, spawn_y, spawn_z)
			}
		end

		# This is the core bit
		@stack.reverse_each_with_index { |character, index|
			progress = (($env[:time] - character.birth_time) / lifetime.to_seconds).clamp(0.0, 1.0)

			# New one comes in behind old one
			with_time_shift(-character.birth_time) {
				# Each successive actor gets a unique child_index
				with_env(:child_index, index + @child_index_offset) {
					with_enter_exit_progress(progress) {
						translate(character.x, character.y, character.z) {
							character.actor.do_render
						}
					}
				}
			}
		}

		# Cleanup, could happen less often than every frame
		while (c=@stack.first and (($env[:time] < c.birth_time) or ($env[:time] > (c.birth_time + lifetime.to_seconds))))
			Character.push(@stack.shift)
			@child_index_offset += 1
		end

		yield
	end
end
