 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class ProjectEffectDirectorCycle < ProjectEffect
	title				'Director Cycle'
	description ''

	setting 'directors', :directors
	setting 'forwards', :event
	setting 'backwards', :event

	setting 'crossfade_time', :timespan

	def crossfade_update(current_spot, desired_spot, time)
		return desired_spot if time.instant?

		current_spot ||= 0.0

		if current_spot != desired_spot
			travel_distance = time.delta		# percent we've travereled through crossfade time is also how far we travel between actors

			if current_spot < desired_spot
				current_spot += travel_distance
				current_spot = desired_spot if current_spot > desired_spot
			else
				current_spot -= travel_distance
				current_spot = desired_spot if current_spot < desired_spot
			end
		end

		return current_spot
	end

	def crossfade_render(first, second, progress)
		with_enter_and_exit(1.0, progress) { first.do_render } if (first and (progress != 1.0))
		with_enter_and_exit(progress, 0.0) { second.do_render } if (second and (progress != 0.0))
	end

	def render
		@current_spot = crossfade_update(@current_spot, (forwards.count - backwards.count), crossfade_time)

		low_index = @current_spot.floor
		crossfade_render(directors.one(low_index), directors.one(low_index+1), (@current_spot - low_index))
	end
end
