 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class VariableInputSliderWithDeadCenter < VariableInput
	title				"Slider with Dead Center"
	description "Reads a slider, and takes values close to 50% to be 50%. Good for inputs that have a physical notch at 50%, but don't always report exactly 50% when there."

	setting 'slider', :slider
	setting 'center_width', :float, :range => 0.05..0.30, :default => 0.05..0.20		# TODO: definite candidate for a non-animating/activating UOSFloat

	# Creates:  _/
	#          /     where 'center width' is width of flat part (out of 1.0 total)
	def value
		half_center_width = (center_width / 2.0)
		ramp_width = (0.5 - half_center_width)

		v = slider
		if v < ramp_width
			# Left of three parts
			(v / ramp_width) * 0.5
		elsif (v > (0.5 + half_center_width))
			# Right of three parts
			0.5 + (((v - 0.5 - half_center_width) / ramp_width) * 0.5)
		else
			0.5
		end
	end
end
