require 'glade_window'
require 'addons_gtk'
require 'vte'
require 'fileutils'

class RenderWindow < GladeWindow
	DEFAULT_VIDEO_FILE_EXTENSION = '.ogv'
	VIDEO_PLAY_EXECUTABLE = 'gnome-open'
	FOLDER_OPEN_EXECUTABLE = 'gnome-open'

	def initialize
		@suggested_filename = ''
		@process_queue = []

		super

		@resolution_combobox.active = 0		# select first option

		@terminal = Vte::Terminal.new.show
		@terminal.signal_connect('child-exited') { on_rendering_done }
		@terminal_container.add(@terminal)
	end

	def project=(project_filepath)
		return unless File.exist?(project_filepath)

		@project_filechooserbutton.filename = project_filepath

		# assume user will load music from the same directory and make it easier for them
		@music_filechooserbutton.current_folder = File.dirname(project_filepath) unless @music_filechooserbutton.filename
	end

	def on_values_changed
		#
		# Do we have all necessary settings?
		#
		input_settings_ready = (@project_filechooserbutton.filename and !@resolution_combobox.active_text.empty?)
#@music_filechooserbutton.filename and
		#
		# Suggest an output filename
		#
		if (input_settings_ready and @output_filename_entry.text.empty?) or (!@suggested_filename.empty? and @output_filename_entry.text == @suggested_filename)
			width, height = @resolution_combobox.active_text.split(' ').first.split('x')
			fps = @frames_per_second_spinbutton.value.to_i

			@suggested_filename = sprintf("%s-%dx%d-%dfps.ogv",
				@music_filechooserbutton.filename ? File.basename(@music_filechooserbutton.filename, '.wav') : File.basename(@project_filechooserbutton.filename, '.luz'),
				width, height, fps)

			@output_filename_entry.text = @suggested_filename
		end

		valid_extensions_array = ['ogv', 'mkv']		#, 'avi', 'mpg', 'mkv']

		#
		# Ready to go?
		#
		if (input_settings_ready and !@output_filename_entry.text.empty?)
			@output_filename = @output_filename_entry.text
			@output_filepath = File.join(@output_folder_filechooserbutton.filename, @output_filename)

			#
			# Refuse to overwrite output file
			#
			if File.exist?(@output_filepath)
				@output_filename_error_label.markup = '<i>A file with that name already exists.</i>'
				@render_button.sensitive = false

			#
			# ffmpeg guesses target format by file extension, so for now that's how specify it
			#
			elsif valid_extensions_array.include?(@output_filepath.split('.').last)
				@output_filename_error_label.markup = ''
				@render_button.sensitive = true
			else
				@output_filename_error_label.markup = "<i>Name must end in #{valid_extensions_array.collect { |s| ".#{s}" }.join(', ')}</i>"
				@render_button.sensitive = false
			end
		else
			@render_button.sensitive = false
		end
	end

	def on_render_button_clicked
		project_path = @project_filechooserbutton.filename
		music_path = @music_filechooserbutton.filename
		fps = @frames_per_second_spinbutton.value.to_i
		width, height = @resolution_combobox.active_text.split(' ').first.split('x')

		@controls_container.visible = false

		#
		# Recording
		#
		command = ("./luz_performer.rb --record --width #{width} --height #{height} --frames-per-second #{fps} \"#{project_path}\"")

		puts "Running: #{command}"
		system(command)

		#
		# Encoding
		#
		@encoding_output_container.visible = true

		# -y = overwrite
		# -r = fps
		# -i = input images
		# -i = input audio
		# -g = keyframe every N frames

		keyframe = '1' #(fps+1) / 2 		# 1=every frame is a keyframe, 2=every other...
		bitrate = '64000k'					# TODO: make this an option
		command = "ffmpeg"
		argv  = ['ffmpeg']
		argv += ['-y']
		argv += ['-r', fps.to_s]
		argv += ['-f', 'image2']
		argv += ['-i', 'frame-%06d.bmp']
		argv += ['-i', music_path] if music_path
		argv += ['-g', keyframe]
		argv += ['-b', bitrate]
		argv += [@output_filepath]

		puts "Running: #{command} #{argv.join(' ')}"

		@terminal.fork_command(command, argv)
	end

	def on_rendering_done
		@terminal.feed("\n\r")
		@terminal.feed("*************************************\n\r")
		@terminal.feed("***   Video Rendering Completed   ***\n\r")
		@terminal.feed("*************************************\n\r")

		@launch_options_container.sensitive = true
	end

	def on_play_button_clicked
		system(VIDEO_PLAY_EXECUTABLE, @output_filepath)
	end

	def on_open_folder_button_clicked
		system(FOLDER_OPEN_EXECUTABLE, @output_folder_filechooserbutton.filename)
	end

	def on_reset_button_clicked
		# Simulate a change so we recheck if the file name exists (it probably does!)
		on_values_changed

		@controls_container.visible = true
		@encoding_output_container.visible = false
		@launch_options_container.sensitive = false
	end

	def hide
		Gtk.main_quit
	end
end
