 ###############################################################################
 #  Copyright 2008 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

require 'style'

class StyleTable
	attr_reader :background_color, :default_style

	def initialize
		@background_color = Color.new([0.0, 0.0, 0.0, 1.0])

		@default_style = Style.new
		@styles = Hash.new
	end

	# Replace
	def with_theme(theme)
		save_all {
			# Overwrite @styles with styles from the theme	NOTE: does not modify/remove named styles that 'theme' is missing
			theme.effects.each { |style|
				@styles[style.title] = style
			}
			@background_color = theme.background_color
			yield
		}
	end

	def with_fade_to_theme(amount, theme)
		save_styles {
			# Overwrite @styles with styles from the theme, appropriately blended
			theme.effects.each { |style|
				starting_style = (@styles[style.title] || @default_style)
				@styles[style.title] = starting_style.dup.set_color(starting_style.color.fade_to(amount, style.color))
			}
			#puts @styles
			yield
		}
	end

	# Replace a named style's color with given color
	def with_style_color(name, color)
		@styles[name] ||= Style.new

		saved_style = @styles[name]

		@styles[name] = saved_style.dup.set_color(color)
		yield
		@styles[name] = saved_style		# Old value or nil
	end

	# NOTE: never returns nil
#	def style_for_name(name)
#		@styles[name] || @default_style
#	end

	def default_color
		@default_style.color
	end

private

	def save_all
		background_color, default_style = @background_color, @default_style
		yield
		@background_color, @default_style = background_color, default_style
	end

	def save_styles
		saved_styles = @styles.dup		# NOTE: copies just the hash, not the things it points to
		yield
		@styles = saved_styles
	end
end
