 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

require 'glade_window', 'tag_treeview'

class TagEditWindow < GladeWindow
	callback :objects_tagged

	def initialize(model)
		super(nil)

		@tag_treeview = TagTreeView.new(model).show
		@tag_treeview_container.add(@tag_treeview)

		# Add "entry completion" to help user by matching what they type against existing tags
		@tag_entry_completion = Gtk::EntryCompletion.new.set_model(model).set_text_column(model.class.name_column_index)
		@tag_entry.set_completion(@tag_entry_completion)

		# Set up our own callbacks for setting the model's checkbox
		@tag_treeview.model.on_is_activated(&method(:tag_is_activated?))
		@tag_treeview.model.on_is_inconsistent(&method(:tag_is_inconsistent?))
		@tag_treeview.on_toggled { |iter|
			tag = @tag_treeview.model.get_object_column(iter)
			if tag_is_activated?(tag)
				remove_tag_from_objects(tag)
			else
				add_tag_to_objects(tag)
			end
			@tag_treeview.model.set_columns_from_object(iter)

			# Notify parent
			objects_tagged_notify(@objects)
		}
		on_key_press(Gdk::Keyval::GDK_Escape) { hide }
	end

	def on_add_button_activate
		# Allow multiple comma separated tags
		@tag_entry.text.split(',').each { |tag|
			tag.strip!
			next if tag.empty?

			add_tag_to_objects(tag)		# NOTE: add to objects before treeview so it shows up checked
			@tag_treeview.select(@tag_treeview.model.add_or_update_object(tag))
		}
		objects_tagged_notify(@objects)

		@tag_entry.text = ''
		@tag_entry.grab_focus
		#hide
	end

	def show_for(objects)
		@objects = objects
		self.title = "Tagging #{@objects.size} object(s)"
		@tag_treeview.update_all
		@tag_entry.text = ''

		present_modal
		@tag_entry.grab_focus
	end

	def add_tag_to_objects(tag)
		@objects.each { |object| object.add_tag(tag) }
	end

	def remove_tag_from_objects(tag)
		@objects.each { |object| object.remove_tag(tag) }
	end

	def tag_is_activated?(tag)
		return (@objects and @objects.all? { |object| object.has_tag?(tag) })
	end

	def tag_is_inconsistent?(tag)
		return (@objects and @objects.inconsistent? { |object| object.has_tag?(tag) })
	end
end
