 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

class Application
	EXCEPTION_REPORT_FORMAT = "#{'#' * 80}\n%s#{'#' * 80}"

	def quit
		@running = false
		Gtk.main_quit
	end

	def do_gc
		was_disabled = GC.enable
		start_time = Time.new.to_f
		GC.start
		#puts "GC took #{Time.new.to_f - start_time}"
		GC.disable if was_disabled
	end

	FRAMES_PER_GC = 200
	def run
		@running = true

		begin
			GC.disable

			frames_since_last_gc = 0
			section('Running Main Loop') {
				Gtk.timeout_remove(@frame_callback) if @frame_callback
				@frame_callback = Gtk.timeout_add(1000.0 / @frames_per_second) {
					$engine.do_frame
					frames_since_last_gc += 1
					if(frames_since_last_gc > FRAMES_PER_GC and gc_now?)
						do_gc
						frames_since_last_gc = 0
					end
					true
				}
				Gtk.main
			}
		rescue Interrupt
			puts 'Caught Ctrl-C Interrupt'
		rescue SystemExit
			puts 'Caught SystemExit'
		rescue Exception => e
			handle_exception(e)
			puts "Retrying..."
			retry
		end
	end

	def gc_now?
		true
	end
end
