module DeepClone
=begin
	# 'list' is the objects already copied (list[old_object] => new_object)
	def deep_clone(list = {})
		return list[self] if list[self]		# Has this object already been cloned?

		# What am I?
		case self
		# We must override Hash and Array clone behavior, which doesn't actually clone contents
		when Hash
			# Make an empty clone to preserve sub-class (self might be a subclass of Hash)
			klone = self.clone
			klone.clear		# NOTE: Keys will be different, so we should start from scratch

			self.each { |k,v| klone[k.deep_clone(list)] = v.deep_clone(list) }
		when Array
			klone = self.clone		# (See comments about Hash above)
			klone.clear
			self.each { |obj| klone << obj.deep_clone(list) }
		else
			begin
				klone = self.clone
			rescue
				# Some objects (Integers, Symbols, etc.) fail cloning so we just reuse them
				klone = self
			end
		end

		# Save, so we don't recurse back to this object (if an instance variable refers to us)
		list[self] = klone

		# Deep clone instance variables
		klone.instance_variables.each { |v|
			klone.instance_variable_set(v, klone.instance_variable_get(v).deep_clone(list))
		}
		return klone
	end
=end

	# This is the same as the above, except it only clones an object if is_cloneable_callback(obj) returns true, otherwise it uses a reference to obj (the root object is always cloned)
	def deep_clone(list = {}, &is_cloneable_callback)
		return list[self] if list[self]
		case self
		when Hash
			klone = self.clone
			klone.clear
			self.each { |k,v| klone[k.deep_clone(list, &is_cloneable_callback)] = v.deep_clone(list, &is_cloneable_callback) }
		when Array
			klone = self.clone
			klone.clear
			self.each { |obj| klone << obj.deep_clone(list, &is_cloneable_callback) }
		else
			begin
				# Determine if we should clone or reuse this object
				is_root_object = (list.empty?)

				if is_root_object or (is_cloneable_callback and is_cloneable_callback.call(self))
					klone = self.clone
				else
					klone = self
				end
			rescue
				klone = self
			end
		end
		list[self] = klone
		klone.instance_variables.each { |v|
			klone.instance_variable_set(v, klone.instance_variable_get(v).deep_clone(list, &is_cloneable_callback))
		}
		return klone
	end
end

if $0 == __FILE__
	require 'test/unit'

	class Object
		include DeepClone
	end

	class TestDeepClone < Test::Unit::TestCase
		def test_sometime
		end
	end
end
