require 'drawing'

class FramebufferObject
	include Drawing

	def initialize(settings={})
		@width, @height = (settings[:width] || 1024), (settings[:height] || 1024)
		@clear_color = [0.0, 0.0, 0.0, 0.0]
		create
	end

private

	def create
		#GL.DeleteFramebuffersEXT(@fbo) if @fbo

		# create FBO
		@fbo = GL.GenFramebuffersEXT(1).first

		# bind it
		GL.BindFramebufferEXT(GL::FRAMEBUFFER_EXT, @fbo)

		#GL.RenderbufferStorageEXT(GL::RENDERBUFFER_EXT, GL::RGBA, @width, @height)

		# create texture to render into
		@image = GL.GenTexture
		GL.BindTexture(GL::TEXTURE_2D, @image)

#		GL.TexParameter(GL::TEXTURE_2D, GL::TEXTURE_WRAP_S, GL::CLAMP) #REPEAT)			# REPEAT mode adds noise to the transparent edge of a texture opposite an opaque edge
#		GL.TexParameter(GL::TEXTURE_2D, GL::TEXTURE_WRAP_T, GL::CLAMP) #REPEAT)
		GL.TexParameter(GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER, GL::LINEAR) #GL::NEAREST)
		GL.TexParameter(GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER, GL::LINEAR) #GL::NEAREST)
		GL.TexImage2D(GL::TEXTURE_2D, 0, GL::RGBA8, @width, @height, 0, GL::RGBA, GL::UNSIGNED_BYTE, nil)

		# attach texture to FBO
		GL.FramebufferTexture2DEXT(GL::FRAMEBUFFER_EXT, GL::COLOR_ATTACHMENT0_EXT, GL::TEXTURE_2D, @image, 0)

#		@depth_buffer = GL.GenRenderbuffersEXT(1).first
#		GL.BindRenderbufferEXT(GL::RENDERBUFFER_EXT, @depth_buffer)
#		GL.RenderbufferStorageEXT(GL::RENDERBUFFER_EXT, GL::DEPTH_COMPONENT, @width, @height)
#		GL.BindRenderbufferEXT(GL::RENDERBUFFER_EXT, 0)
		# attach to depth attachment point
#		GL.FramebufferRenderbufferEXT(GL::FRAMEBUFFER_EXT, GL::DEPTH_ATTACHMENT_EXT, GL::RENDERBUFFER_EXT, @depth_buffer)

		# check status
		if (status = GL.CheckFramebufferStatusEXT(GL::FRAMEBUFFER_EXT)) != GL::FRAMEBUFFER_COMPLETE_EXT
			puts "framebuffer creation failed (status: #{status})"
			@failed
		end
		GL.BindFramebufferEXT(GL::FRAMEBUFFER_EXT, 0)
	end

public

	def using
		$fbo_stack ||= []
		# Switch rendering to FBO
		$fbo_stack << @fbo
		GL.BindFramebufferEXT(GL::FRAMEBUFFER_EXT, $fbo_stack.last)

		GL.PushAttrib(GL::VIEWPORT_BIT)
		GL.Viewport(0, 0, @width, @height)

		with_identity_transformation {
			clear_screen(@clear_color)
			yield
		}

		GL.PopAttrib
		$fbo_stack.pop
		GL.BindFramebufferEXT(GL::FRAMEBUFFER_EXT, $fbo_stack.last || 0)
	end

	def with_image(&proc)
		with_texture_scale(1.0, -1.0) {		# texture coordinates seem to need to be flipped vertically with FBOs
			with_texture(@image, &proc)
		}
	end
end
