 ###############################################################################
 #  Copyright 2007 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

# Mixin: EasyDSL simplifies creating Domain Specific Languages.

# Provides class-level methods for defining other class-level methods.

module Memoize
   MEMOIZE_VERSION = "1.2.3"
   
   # Memoize the method +name+.  If +file+ is provided, then the method results
   # are stored on disk as well as in memory.
   def memoize(name)
      cache = {}

      (class<<self; self; end).send(:define_method, name) do |*args|
         unless cache.has_key?(args)
            cache[args] = super
         end
         cache[args]
      end
      cache
   end
end

if $0 == __FILE__
	require 'test/unit'
	require 'benchmark'
    
	class TestMemoize < Test::Unit::TestCase
		class Curve
			include Memoize
			SIZE = 1000

			def initialize
				@approximation = (0..SIZE).to_a.collect { |n| n / (SIZE-1.0) }
			end

			def resolve_approximation(x)		# Expects 0.0->1.0
				step_size = (1.0 / (@approximation.size - 1))

				ia = (x * (@approximation.size - 1)).floor.to_i
				ib = (ia + 1) % (@approximation.size)

				percent_between = (x - (ia * step_size)) / step_size

				@approximation[ia] + (percent_between * (@approximation[ib] - @approximation[ia]))
			end
		end

		def test_speedup
			c = Curve.new

			n = 200000
			inputs = (1..n).to_a.collect { rand }

			Benchmark.bm do |bm|
				bm.report("   without memoize:") { for i in 1...n ; c.resolve_approximation(inputs[i]) ; end }
				c.memoize(:value)
				bm.report("      with memoize:") { for i in 1...n ; c.resolve_approximation(inputs[i]) ; end }
				bm.report("again with memoize:") { for i in 1...n ; c.resolve_approximation(inputs[i]) ; end }
			end
		end
	end
end
