 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

# ObjectCounterStore stores an integer count on behalf of another object.

# Stores data in a hash, indexed by the object's id, to make get and set options fast.

require 'object_data_store'

class ObjectCounterStore < ObjectDataStore
	def initialize
		super(0)
	end

	def increment(object)
		set(object, get(object) + 1)
	end

	def decrement(object)
		count = get(object)
		set(object, count - 1) if count > 0
	end

	def decrement_with_remove(object)
		count = get(object)
		if count > 1
			set(object, count - 1)
		else		# 1 or 0 or non-existent
			delete(object)
		end
	end
end

if $0 == __FILE__
	require 'test/unit'

	class TestObjectCounterStore < Test::Unit::TestCase
  	def test_increment_and_decrement
			store = ObjectCounterStore.new
			obj1 = Object.new

			# Incrementing
			assert_equal 0, store[obj1]

			store.increment(obj1)
			assert_equal 1, store[obj1]

			store.increment(obj1)
			assert_equal 2, store[obj1]

			# Decrementing
			store.decrement(obj1)
			assert_equal 1, store[obj1]

			store.decrement(obj1)
			assert_equal 0, store[obj1]

			store.decrement(obj1)
			assert_equal 0, store[obj1]		# Doesn't go below 0
  	end

		def test_decrement_with_remove
			store = ObjectCounterStore.new
			obj1 = Object.new
			
			# Increment
			store.increment(obj1)
			assert_equal 1, store[obj1]
			assert_equal 1, store.size

			store.increment(obj1)
			assert_equal 2, store[obj1]
			assert_equal 1, store.size
			
			# Decrement with remove
			store.decrement_with_remove(obj1)
			assert_equal 1, store[obj1]
			assert_equal 1, store.size

			store.decrement_with_remove(obj1)
			assert_equal 0, store[obj1]
			assert_equal 0, store.size
		end
	end
end
