 ###############################################################################
 #  Copyright 2006 Ian McIntosh <ian@openanswers.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU Library General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ###############################################################################

# ObjectDataStore stores any object ON BEHALF of another object.

# Stores data in a hash, indexed by the object's id, to make get and set options fast.

class ObjectDataStore
	def initialize(default_value=nil)
		@hash = Hash.new(default_value)
	end

	def get(object)
		@hash[object.object_id]
	end
	alias :[] :get

	def set(object, value)
		existing = @hash.has_key?(object.object_id)
		@hash[object.object_id] = value
		ObjectSpace.define_finalizer(object, lambda { |object_id| @hash.delete(object_id) }) unless existing
	end
	alias :[]= :set

	def delete(object)
		@hash.delete(object.object_id)
	end
	alias :remove :delete
	
	def size
		@hash.size
	end
end

if $0 == __FILE__
	require 'test/unit'

	class TestObjectDataStore < Test::Unit::TestCase
  	def test_add_and_remove
			store = ObjectDataStore.new(default_value = 999)
			obj1 = Object.new
			obj2 = Object.new

			# Storing
			store.set(obj1, 111)

			# Retrieving
			assert_equal 111, store[obj1]
			assert_equal 111, store.get(obj1)

			# Default values
			assert_equal 999, store[obj2]

			# Remove restores default value and frees storage
			store.remove(obj1)
			assert_equal 999, store[obj1]
			assert_equal 0, store.size
  	end
	end
end
