#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//
// WiiMote
//
#define _ENABLE_TILT
#define _ENABLE_FORCE

#include <wiimote.h>
#include <wiimote_api.h>

//
// liblo OpenSoundControl (http://liblo.sourceforge.net/)
//
#include <lo/lo.h>

#define UDP_PORT ("10007") // LOOOZ :D
#define UDP_ADDRESS NULL //("255.255.255.255")

#define ADDRESS_BUFFER_SIZE 1000
char address_buffer[ADDRESS_BUFFER_SIZE + 1];

float clamp_float(float value, float min, float max)
{
	return ((value >= max) ? max : ((value <= min) ? min : value));
}

typedef struct
{
	int min;
	int max;
} TLimits;

float scale_and_expand_limits(int value, TLimits* limits)
{
	if(value < limits->min)
		limits->min = value;

	if(value > limits->max)
		limits->max = value;

	int range = (limits->max - limits->min);

	if(range == 0)
		return 0.0;

	return (float)(value - limits->min) / (float)range;
}

#define BATTERY_LOW (20)
#define FRAMES_PER_BLINK (40)
#define MAX_FORCE (3.0)

int main(int argc, char *argv[])
{
	wiimote_t wiimote = WIIMOTE_INIT;
	wiimote_t old_wiimote = WIIMOTE_INIT;
	int wiimote_index = 0;		// This should be command-line option across all miniapps
	int i;
	float last_known_good_tilt_y = 0.0;

	lo_address t = lo_address_new(UDP_ADDRESS, UDP_PORT);

	printf("Put Wiimote in discovery mode now (press 1 + 2 at the same time)...\n");

	while(wiimote_discover(&wiimote, 1) <= 0) {
		// Uh, try again?
	}

	if (wiimote_connect(&wiimote, wiimote.link.r_addr) < 0) {
		fprintf(stderr, "Failed to open Wiimote: %s\n", wiimote_get_error());
	}
	printf("Connected to Wiimote %02d...\n", wiimote_index + 1);

	// Enable features
	wiimote.mode.acc = 1;
	wiimote.mode.ir = 1;		// see comment in input-manager/input-wiimote.cc

	// Default LED state reflects the device number ("Wiimote 01" in Luz will have 1st bit set, "Wiimote 02" the second, etc.)
	int led_state = 1 << wiimote_index;

	printf("Ctrl-C to quit...\n");
	int frame_number = 0;
	while (1) {
		frame_number++;

		if(wiimote_update(&wiimote) < 0) {
			wiimote_disconnect(&wiimote);
			fprintf(stderr, "Lost connection to Wiimote.\n");
			exit(1);
		}

		//
		// Buttons
		//
		struct { int mask; char* name; } wiimote_buttons[] = {
			{WIIMOTE_KEY_1, "1"},
			{WIIMOTE_KEY_2, "2"},
			{WIIMOTE_KEY_A, "A"},
			{WIIMOTE_KEY_B, "B"},
			{WIIMOTE_KEY_PLUS, "+"},
			{WIIMOTE_KEY_MINUS, "-"},
			{WIIMOTE_KEY_HOME, "Home"},	// ⌂
			{WIIMOTE_KEY_LEFT, "Left"},	// ◀
			{WIIMOTE_KEY_RIGHT, "Right"},	// ▶
			{WIIMOTE_KEY_UP, "Up"},	// ▲
			{WIIMOTE_KEY_DOWN, "Down"}	// ▼
		};

		for(i=0 ; i < (sizeof(wiimote_buttons) / sizeof(wiimote_buttons[0])) ; i++) {
			if((wiimote.keys.bits & wiimote_buttons[i].mask) != (old_wiimote.keys.bits & wiimote_buttons[i].mask)) {
				snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "Wiimote %02d / %s", wiimote_index + 1, wiimote_buttons[i].name);
				lo_send(t, address_buffer, "i", (wiimote.keys.bits & wiimote_buttons[i].mask) > 0 ? 1 : 0);
			}
		}

		//
		// IR (x,y,z)
		//

		//
		// Pitch and Roll
		//

		// Clamp to -1.0..1.0 g-force, because any more isn't a function of tilt but rather moving the Wiimote.
		float pitch = clamp_float(((float)(wiimote.axis.y - wiimote.cal.y_zero) / (float)(wiimote.cal.y_scale - wiimote.cal.y_zero)), -1.0, 1.0) ;
		pitch *= -1.0;		// We want positive values when Wiimote is vertical
		pitch += 1.0;			// -1.0..1.0 to 0.0..2.0
		pitch /= 2.0;			//  0.0..2.0 to 0.0..1.0
		snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "Wiimote %02d / Pitch", wiimote_index + 1);
		lo_send(t, address_buffer, "f", pitch);

		// Clamp to -1.0..1.0 g-force, because any more isn't a function of tilt but rather moving the Wiimote.
		float roll = clamp_float(((float)(wiimote.axis.x - wiimote.cal.x_zero) / (float)(wiimote.cal.x_scale - wiimote.cal.x_zero)), -1.0, 1.0) ;
		roll += 1.0;		// -1.0..1.0 to 0.0..2.0
		roll /= 2.0;		//  0.0..2.0 to 0.0..1.0
		snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "Wiimote %02d / Roll", wiimote_index + 1);
		lo_send(t, address_buffer, "f", roll);

		// Force
		//static TLimits force_z_limits = {10000,-10000};
		//float force_z = scale_and_expand_limits(wiimote.axis.z, &force_z_limits);
		//printf("%f,%f,%f\n", wiimote.axis.x, wiimote.axis.y, wiimote.axis.z);
		//float force = sqrt(wiimote.axis.x * wiimote.axis.x + wiimote.axis.y * wiimote.axis.y + wiimote.axis.z * wiimote.axis.z);
		//printf("force_z = %f\n", force_z);
		//snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "Wiimote %02d / Force Z", wiimote_index + 1);
		//lo_send(t, address_buffer, "f", force_z);

		// Possibly useful later: Wiimote buttons are facing the sky when: (wiimote.axis.z - wiimote.cal.z_zero) >= 0
		wiimote_copy(&wiimote, &old_wiimote);

		// Blink LED(s) when battery low
		int blink = ((wiimote.battery < BATTERY_LOW) && (((frame_number / FRAMES_PER_BLINK) % 2) == 0));
		wiimote.led.bits = (blink) ? 0 : led_state;

		usleep(5 * 1000);
	}

/*
 		if(acc_x != last_acc_x) {
			snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "WiiMote 1 / Acceleration X");
			lo_send(t, address_buffer, "f", acc_x);
			last_acc_x = acc_x;
		}

		if(acc_y != last_acc_y) {
			snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "WiiMote 1 / Acceleration Y");
			lo_send(t, address_buffer, "f", acc_y);
			last_acc_y = acc_y;
		}

		if(acc_z != last_acc_z) {
			snprintf(address_buffer, ADDRESS_BUFFER_SIZE, "WiiMote 1 / Acceleration Z");
			lo_send(t, address_buffer, "f", acc_z);
			last_acc_z = acc_z;
		}
*/
}

/* Prototype cwiid_callback with cwiid_callback_t, define it with the actual
 * type - this will cause a compile error (rather than some undefined bizarre
 * behavior) if cwiid_callback_t changes */
/* cwiid_mesg_callback_t has undergone a few changes lately, hopefully this
 * will be the last.  Some programs need to know which messages were received
 * simultaneously (e.g. for correlating accelerometer and IR data), and the
 * sequence number mechanism used previously proved cumbersome, so we just
 * pass an array of messages, all of which were received at the same time.
 * The id is to distinguish between multiple wiimotes using the same callback.
 * */

/*
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count,
                    union cwiid_mesg mesg[], struct timespec *timestamp)
{
	int i, j;
	int valid_source;
	float value, zero_point;

static TLimits acc_x_limits = {10000,-10000};
static TLimits acc_y_limits = {10000,-10000};
static TLimits acc_z_limits = {10000,-10000};

	for (i=0; i < mesg_count; i++)
	{
		switch (mesg[i].type) {
		case CWIID_MESG_STATUS:
			printf("Battery at %02.1f%%\n", 100.0 * (mesg[i].status_mesg.battery / 255.0));
			break;
		case CWIID_MESG_BTN:
			printf("Button Report: %.4X\n", mesg[i].btn_mesg.buttons);
			break;
		case CWIID_MESG_ACC:
			value = scale_and_expand_limits(mesg[i].acc_mesg.acc[CWIID_X], &acc_x_limits);

			if(acc_x_limits.max != accerometer_calibration.zero[CWIID_X]) {		// Prevent dividision by 0.  This might happen before properly expanding the limit.
				zero_point = ((float)accerometer_calibration.zero[CWIID_X] / (float)acc_x_limits.max);

				if(value >= zero_point) {
					acc_x = 0.5 + ((value - zero_point) / (1.0 - zero_point));
				}
				else {
					acc_x = 0.5 * (value / zero_point);
				}
			}

			value = scale_and_expand_limits(mesg[i].acc_mesg.acc[CWIID_Y], &acc_y_limits);
			if(acc_y_limits.max != accerometer_calibration.zero[CWIID_Y]) {		// Prevent dividision by 0.  This might happen before properly expanding the limit.
				zero_point = ((float)accerometer_calibration.zero[CWIID_Y] / (float)acc_y_limits.max);

				if(value >= zero_point) {
					acc_y = 0.5 + ((value - zero_point) / (1.0 - zero_point));
				}
				else {
					acc_y = 0.5 * (value / zero_point);
				}
			}

			value = scale_and_expand_limits(mesg[i].acc_mesg.acc[CWIID_Z], &acc_z_limits);
			if(acc_z_limits.max != accerometer_calibration.one[CWIID_Z]) {		// Prevent dividision by 0.  This might happen before properly expanding the limit.
				zero_point = ((float)accerometer_calibration.one[CWIID_Z] / (float)acc_z_limits.max);

				if(value >= zero_point) {
					acc_z = 0.5 + ((value - zero_point) / (1.0 - zero_point));
				}
				else {
					acc_z = 0.5 * (value / zero_point);
				}
			}
			break;
		case CWIID_MESG_IR:
			//printf("IR Report: ");
			valid_source = 0;
			for (j = 0; j < CWIID_IR_SRC_COUNT; j++) {
				if (mesg[i].ir_mesg.src[j].valid) {
					valid_source = 1;
					printf("IR Report (%d,%d)\n", mesg[i].ir_mesg.src[j].pos[CWIID_X],
					                   mesg[i].ir_mesg.src[j].pos[CWIID_Y]);
				}
			}
			if (!valid_source) {
				//printf("no sources detected");
			}
			//printf("\n");
			break;
		case CWIID_MESG_NUNCHUK:
			printf("Nunchuk Report: btns=%.2X stick=(%d,%d) acc.x=%d acc.y=%d "
			       "acc.z=%d\n", mesg[i].nunchuk_mesg.buttons,
			       mesg[i].nunchuk_mesg.stick[CWIID_X],
			       mesg[i].nunchuk_mesg.stick[CWIID_Y],
			       mesg[i].nunchuk_mesg.acc[CWIID_X],
			       mesg[i].nunchuk_mesg.acc[CWIID_Y],
			       mesg[i].nunchuk_mesg.acc[CWIID_Z]);
			break;
		case CWIID_MESG_CLASSIC:
			printf("Classic Report: btns=%.4X l_stick=(%d,%d) r_stick=(%d,%d) "
			       "l=%d r=%d\n", mesg[i].classic_mesg.buttons,
			       mesg[i].classic_mesg.l_stick[CWIID_X],
			       mesg[i].classic_mesg.l_stick[CWIID_Y],
			       mesg[i].classic_mesg.r_stick[CWIID_X],
			       mesg[i].classic_mesg.r_stick[CWIID_Y],
			       mesg[i].classic_mesg.l, mesg[i].classic_mesg.r);
			break;
		case CWIID_MESG_ERROR:
			if (cwiid_close(wiimote)) {
				fprintf(stderr, "Error on wiimote disconnect\n");
				exit(-1);
			}
			exit(0);
			break;
		default:
			printf("Unknown Report");
			break;
		}
	}
}
*/

/*
typedef struct
{
	float min;
	float max;
} TLimitsFloat;

float scale_and_expand_limits_float(float value, TLimitsFloat* limits)
{
	if(value < limits->min)
		limits->min = value;

	if(value > limits->max)
		limits->max = value;

	int range = (limits->max - limits->min);

	if(range == 0.0)
		return 0.0;

	return (float)(value - limits->min) / (float)range;
}
*/

/*
typedef struct
{
	int min;
	int max;
} TLimits;

float scale_and_expand_limits(int value, TLimits* limits)
{
	if(value < limits->min)
		limits->min = value;

	if(value > limits->max)
		limits->max = value;

	int range = (limits->max - limits->min);

	if(range == 0)
		return 0.0;

	return (float)(value - limits->min) / (float)range;
}
*/
